from enum import Enum
from random import random as _random

from pyqrbtf.utils.center_module import CenterModule
from .utils.type_table import get_type_table, QRPointType


class ContentPointType(str, Enum):
    square = "square"
    circle = "circle"


def rand(start: float, end: float) -> float:
    return start + (end - start) * _random()


class DotsDrawer(CenterModule):
    def __init__(
        self,
        content_point_type: ContentPointType,
        content_point_scale: int = 0,
        content_point_opacity: int = 1,
        **kwargs,
    ):
        self.content_point_type = content_point_type

        self.content_point_scale = content_point_scale
        self.content_point_opacity = content_point_opacity
        super().__init__(**kwargs)

    def draw(self, matrix, colors):
        type_table = get_type_table(matrix)

        points = []

        content_point_size = self.content_point_scale * 1.01
        content_point_size_half = content_point_size / 2.0
        content_point_offset = (1 - content_point_size) / 2.0

        rect = self.rect
        circle = self.circle

        for y in range(len(matrix)):
            for x in range(len(matrix)):
                if not matrix[x][y]:
                    continue
                match type_table[x][y]:
                    case QRPointType.POS_OTHER:
                        continue
                    case QRPointType.POS_CENTER:
                        self.draw_center(points, colors, x, y)
                    case _:
                        color = colors.get(
                            type_table[x][y], colors.get(QRPointType.DATA, "#000")
                        )
                        is_constant = content_point_size > 0
                        if self.content_point_type == ContentPointType.square:
                            size = is_constant if content_point_size else rand(0.3, 1)
                            offset = (
                                is_constant
                                if content_point_offset
                                else (1 - size) / 2.0
                            )
                            points.append(
                                rect(
                                    opacity=self.content_point_opacity,
                                    width=size,
                                    height=size,
                                    fill=color,
                                    x=x + offset,
                                    y=y + offset,
                                ),
                            )
                        elif self.content_point_type == ContentPointType.circle:
                            half_size = (
                                is_constant
                                if content_point_size_half
                                else rand(0.3, 1) / 2.0
                            )
                            points.append(
                                circle(
                                    opacity=self.content_point_opacity,
                                    r=half_size,
                                    fill=color,
                                    cx=x + 0.5,
                                    cy=y + 0.5,
                                ),
                            )
        return points



from xml.etree import ElementTree as ET


class Figure:
    id = 0

    @staticmethod
    def _attrib(**kwargs):
        return {k.replace("_", "-"): str(v) for k, v in kwargs.items() if v is not None}

    def rect(self, **kwargs):
        el = ET.Element(
            "rect",
            id=str(self.id),
            attrib=self._attrib(**kwargs),
        )
        self.id += 1
        return el

    def circle(self, **kwargs):
        el = ET.Element(
            "circle",
            id=str(self.id),
            attrib=self._attrib(**kwargs),
        )
        self.id += 1
        return el

    def path(self, **kwargs):
        el = ET.Element(
            "path",
            id=str(self.id),
            attrib=self._attrib(**kwargs),
        )
        self.id += 1
        return el

    def line(self, **kwargs):
        el = ET.Element(
            "line",
            id=str(self.id),
            attrib=self._attrib(**kwargs),
        )
        self.id += 1
        return el



from enum import Enum


class QRPointType(int, Enum):
    NONE = -1
    DATA = 0
    POS_CENTER = 1
    POS_OTHER = 2
    ALIGN_CENTER = 3
    ALIGN_OTHER = 4
    TIMING = 5
    FORMAT = 6
    VERSION = 7


def get_type_table(qrcode: list[list[bool]]) -> list[list[QRPointType | None]]:
    n_count = len(qrcode)
    pd = [[3, 3], [3, n_count - 4], [n_count - 4, 3]]

    type_table: list[list[QRPointType | None]] = [
        [None for _ in range(n_count)] for _ in range(n_count)
    ]

    for i in range(8, n_count - 7):
        type_table[i][6] = QRPointType.TIMING
        type_table[6][i] = QRPointType.TIMING

    for pos in qrcode:
        type_table[pos[0]][pos[1]] = QRPointType.ALIGN_CENTER
        for r in range(-2, 3):
            for c in range(-2, 3):
                if not (r == 0 and c == 0):
                    type_table[pos[0] + r][pos[1] + c] = QRPointType.ALIGN_OTHER

    for point in pd:
        type_table[point[0]][point[1]] = QRPointType.POS_CENTER
        for r in range(-4, 5):
            for c in range(-4, 5):
                if (
                    0 <= point[0] + r < n_count
                    and 0 <= point[1] + c < n_count
                    and not (r == 0 and c == 0)
                ):
                    type_table[point[0] + r][point[1] + c] = QRPointType.POS_OTHER

    for i in range(9):
        if i != 6:
            type_table[i][8] = QRPointType.FORMAT
            type_table[8][i] = QRPointType.FORMAT
        if i < 7:
            type_table[n_count - i - 1][8] = QRPointType.FORMAT
        if i < 8:
            type_table[8][n_count - i - 1] = QRPointType.FORMAT

    for i in range(n_count - 11, n_count - 8):
        for j in range(6):
            type_table[i][j] = QRPointType.VERSION
            type_table[j][i] = QRPointType.VERSION

    for i in range(n_count):
        for j in range(n_count):
            if type_table[i][j] is None:
                type_table[i][j] = QRPointType.DATA

    return type_table



from xml.etree import ElementTree as ET

from .figure import Figure
from .type_table import QRPointType


class BaseDrawer:
    size = 500

    def __init__(self):
        self._figure = Figure()
        self.rect = self._figure.rect
        self.circle = self._figure.circle
        self.path = self._figure.path
        self.line = self._figure.line

    def to_bytes(
        self, matrix: list[list[bool]], colors: dict[QRPointType, str]
    ) -> bytes:
        return ET.tostring(self.svg(matrix, colors), encoding="utf-8")

    def svg(
        self, matrix: list[list[bool]], colors: dict[QRPointType, str]
    ) -> ET.Element:
        assert len(matrix) == len(matrix[0])

        n_count = len(matrix)

        view_box = [
            -n_count / 10,
            -n_count / 10,
            n_count + n_count / 10 * 2,
            n_count + n_count / 10 * 2,
        ]

        attrib = {
            "xmlns": "http://www.w3.org/2000/svg",
            "width": str(self.size),
            "height": str(self.size),
            "viewBox": " ".join(map(str, view_box)),
            "class": "w-full bg-white",
        }

        et = ET.Element(
            "svg",
            attrib=attrib,
        )
        if background := colors.get(QRPointType.NONE):
            et.append(
                ET.Element(
                    "rect",
                    width="100%",
                    height="100%",
                    x=str(view_box[0]),
                    y=str(view_box[1]),
                    fill=background,
                )
            )
        for i in self.draw(matrix, colors):
            et.append(i)

        return et

    def draw(
        self, matrix: list[list[bool]], colors: dict[QRPointType, str]
    ) -> list[ET.Element]:
        raise NotImplementedError



from random import random

from .utils.center_module import CenterModule
from .utils.figure import Figure
from .utils.type_table import get_type_table, QRPointType


def rand(min_: float, max_: float) -> float:
    return random() * (max_ - min_) + min_


class CirclesDrawer(CenterModule):
    def draw(self, matrix, colors):
        size = len(matrix)
        type_table = get_type_table(matrix)
        point_list = []
        g1 = []
        g2 = []

        background_color = colors.get(QRPointType.NONE, "none")
        other_color = colors.get(QRPointType.DATA, "#000000")

        available = [[True] * size for _ in range(size)]
        ava2 = [[True] * size for _ in range(size)]

        f = Figure()
        circle = f.circle

        for y in range(size):
            for x in range(size):
                if type_table[x][y] == QRPointType.POS_OTHER:
                    continue
                elif type_table[x][y] == QRPointType.POS_CENTER:
                    self.draw_center(point_list, colors, x, y)
                else:
                    if available[x][y] and ava2[x][y] and x < size - 2 and y < size - 2:
                        ctn = True
                        for i in range(3):
                            for j in range(3):
                                if not ava2[x + i][y + j]:
                                    ctn = False
                        if (
                            ctn
                            and matrix[x + 1][y]
                            and matrix[x + 1][y + 2]
                            and matrix[x][y + 1]
                            and matrix[x + 2][y + 1]
                        ):
                            g2.append(
                                circle(
                                    cx=x + 1 + 0.5,
                                    cy=y + 1 + 0.5,
                                    r=1,
                                    fill=background_color,
                                    stroke=other_color,
                                    stroke_width=rand(0.33, 0.6),
                                )
                            )
                            if matrix[x + 1][y + 1]:
                                g1.append(
                                    circle(
                                        r=0.5 * rand(0.5, 1),
                                        fill=other_color,
                                        cx=x + 1 + 0.5,
                                        cy=y + 1 + 0.5,
                                    )
                                )
                            available[x + 1][y] = False
                            available[x][y + 1] = False
                            available[x + 2][y + 1] = False
                            available[x + 1][y + 2] = False
                            for i in range(3):
                                for j in range(3):
                                    ava2[x + i][y + j] = False

                    if x < size - 1 and y < size - 1:
                        if (
                            matrix[x][y]
                            and matrix[x + 1][y]
                            and matrix[x][y + 1]
                            and matrix[x + 1][y + 1]
                        ):
                            g1.append(
                                circle(
                                    cx=x + 1,
                                    cy=y + 1,
                                    r=0.5**0.5,
                                    fill=background_color,
                                    stroke=other_color,
                                    stroke_width=rand(0.33, 0.6),
                                )
                            )
                            for i in range(2):
                                for j in range(2):
                                    available[x + i][y + j] = False
                                    ava2[x + i][y + j] = False

                    if available[x][y] and y < size - 1:
                        if matrix[x][y] and matrix[x][y + 1]:
                            point_list.append(
                                circle(
                                    cx=x + 0.5,
                                    cy=y + 1,
                                    r=0.5 * rand(0.95, 1.05),
                                    fill=background_color,
                                    stroke=other_color,
                                    stroke_width=rand(0.36, 0.4),
                                )
                            )
                            available[x][y] = False
                            available[x][y + 1] = False

                    if available[x][y] and x < size - 1:
                        if matrix[x][y] and matrix[x + 1][y]:
                            point_list.append(
                                circle(
                                    cx=x + 1,
                                    cy=y + 0.5,
                                    r=0.5 * rand(0.95, 1.05),
                                    fill=background_color,
                                    stroke=other_color,
                                    stroke_width=rand(0.36, 0.4),
                                )
                            )
                            available[x][y] = False
                            available[x + 1][y] = False

                    if available[x][y]:
                        if matrix[x][y]:
                            point_list.append(
                                circle(
                                    cx=x + 0.5,
                                    cy=y + 0.5,
                                    r=0.5 * rand(0.5, 1),
                                    fill=other_color,
                                )
                            )

                        elif type_table[x][y] == QRPointType.DATA:
                            if rand(0, 1) > 0.85:
                                g2.append(
                                    circle(
                                        r=0.5 * rand(0.85, 1.3),
                                        cx=x + 0.5,
                                        cy=y + 0.5,
                                        fill=background_color,
                                        stroke=other_color,
                                        stroke_width=rand(0.15, 0.33),
                                    )
                                )

        point_list.extend(g1)
        point_list.extend(g2)

        return point_list



from enum import Enum
from random import random as _random

from pyqrbtf.utils.center_module import CenterModule
from .utils.type_table import get_type_table, QRPointType


class ContentLineType(str, Enum):
    radial = "radial"
    horizontal = "horizontal"
    vertical = "vertical"
    tl_br = "tl_br"
    tr_bl = "tr_bl"
    cross = "cross"
    interlock = "interlock"


def random(start: float, end: float) -> float:
    return start + (end - start) * _random()


class LinesDrawer(CenterModule):
    def __init__(
        self,
        content_line_type: ContentLineType,
        content_point_scale=0.5,
        content_point_opacity=1,
        **kwargs,
    ):
        self.content_line_type = content_line_type

        self.content_point_scale = content_point_scale
        self.content_point_opacity = content_point_opacity
        super().__init__(**kwargs)

    def draw(self, matrix, colors):
        type_table = get_type_table(matrix)

        points = []

        n_count = len(matrix)

        opacity = self.content_point_opacity
        size = self.content_point_scale

        available = [[True] * n_count for _ in range(n_count)]
        ava2 = [[True] * n_count for _ in range(n_count)]

        line = self.line
        circle = self.circle

        for y in range(n_count):
            for x in range(n_count):
                if not matrix[x][y]:
                    continue
                match type_table[x][y]:
                    case QRPointType.POS_OTHER:
                        continue
                    case QRPointType.POS_CENTER:
                        self.draw_center(points, colors, x, y)
                    case _:
                        color = colors.get(
                            type_table[x][y], colors.get(QRPointType.DATA, "#000")
                        )
                        if self.content_line_type == ContentLineType.horizontal:
                            if x == 0 or (
                                x > 0 and (not matrix[x - 1][y] or not ava2[x - 1][y])
                            ):
                                start = 0
                                end = 0
                                ctn = True
                                while ctn and x + end < n_count:
                                    if matrix[x + end][y] and ava2[x + end][y]:
                                        end += 1
                                    else:
                                        ctn = False

                                if end - start > 1:
                                    for i in range(start, end):
                                        ava2[x + i][y] = False
                                        available[x + i][y] = False

                                    points.append(
                                        line(
                                            opacity=opacity,
                                            x1=x + 0.5,
                                            y1=y + 0.5,
                                            x2=x + end - start - 0.5,
                                            y2=y + 0.5,
                                            stroke_width=size,
                                            stroke=color,
                                            stroke_linecap="round",
                                        ),
                                    )

                            if available[x][y]:
                                points.append(
                                    circle(
                                        opacity=opacity,
                                        r=size / 2.0,
                                        fill=color,
                                        cx=x + 0.5,
                                        cy=y + 0.5,
                                    ),
                                )

                        if self.content_line_type == ContentLineType.vertical:
                            if y == 0 or (
                                y > 0 and (not matrix[x][y - 1] or not ava2[x][y - 1])
                            ):
                                start = 0
                                end = 0
                                ctn = True
                                while ctn and y + end < n_count:
                                    if matrix[x][y + end] and ava2[x][y + end]:
                                        end += 1
                                    else:
                                        ctn = False

                                if end - start > 1:
                                    for i in range(start, end):
                                        ava2[x][y + i] = False
                                        available[x][y + i] = False

                                    points.append(
                                        line(
                                            opacity=opacity,
                                            x1=x + 0.5,
                                            y1=y + 0.5,
                                            x2=x + 0.5,
                                            y2=y + end - start - 1 + 0.5,
                                            stroke_width=size,
                                            stroke=color,
                                            stroke_linecap="round",
                                        ),
                                    )

                            if available[x][y]:
                                points.append(
                                    circle(
                                        opacity=opacity,
                                        r=size / 2.0,
                                        fill=color,
                                        cx=x + 0.5,
                                        cy=y + 0.5,
                                    ),
                                )

                        if self.content_line_type == ContentLineType.interlock:
                            if y == 0 or (
                                y > 0 and (not matrix[x][y - 1] or not ava2[x][y - 1])
                            ):
                                start = 0
                                end = 0
                                ctn = True
                                while ctn and y + end < n_count:
                                    if (
                                        matrix[x][y + end]
                                        and ava2[x][y + end]
                                        and end - start <= 3
                                    ):
                                        end += 1
                                    else:
                                        ctn = False

                                if end - start > 1:
                                    for i in range(start, end):
                                        ava2[x][y + i] = False
                                        available[x][y + i] = False

                                    points.append(
                                        line(
                                            opacity=opacity,
                                            x1=x + 0.5,
                                            y1=y + 0.5,
                                            x2=x + 0.5,
                                            y2=y + end - start - 1 + 0.5,
                                            stroke_width=size,
                                            stroke=color,
                                            stroke_linecap="round",
                                        ),
                                    )

                            if x == 0 or (
                                x > 0 and (not matrix[x - 1][y] or not ava2[x - 1][y])
                            ):
                                start = 0
                                end = 0
                                ctn = True
                                while ctn and x + end < n_count:
                                    if (
                                        matrix[x + end][y]
                                        and ava2[x + end][y]
                                        and end - start <= 3
                                    ):
                                        end += 1
                                    else:
                                        ctn = False

                                if end - start > 1:
                                    for i in range(start, end):
                                        ava2[x + i][y] = False
                                        available[x + i][y] = False

                                    points.append(
                                        line(
                                            opacity=opacity,
                                            x1=x + 0.5,
                                            y1=y + 0.5,
                                            x2=x + end - start - 0.5,
                                            y2=y + 0.5,
                                            stroke_width=size,
                                            stroke=color,
                                            stroke_linecap="round",
                                        ),
                                    )

                            if available[x][y]:
                                points.append(
                                    circle(
                                        opacity=opacity,
                                        r=size / 2.0,
                                        fill=color,
                                        cx=x + 0.5,
                                        cy=y + 0.5,
                                    ),
                                )

                        if self.content_line_type == ContentLineType.radial:
                            if x > y != x + y < n_count:
                                if y == 0 or (
                                    y > 0
                                    and (not matrix[x][y - 1] or not ava2[x][y - 1])
                                ):
                                    start = 0
                                    end = 0
                                    ctn = True
                                    while ctn and y + end < n_count:
                                        if (
                                            matrix[x][y + end]
                                            and ava2[x][y + end]
                                            and end - start <= 3
                                        ):
                                            end += 1
                                        else:
                                            ctn = False

                                    if end - start > 1:
                                        for i in range(start, end):
                                            ava2[x][y + i] = False
                                            available[x][y + i] = False

                                        points.append(
                                            line(
                                                opacity=opacity,
                                                x1=x + 0.5,
                                                y1=y + 0.5,
                                                x2=x + 0.5,
                                                y2=y + end - start - 1 + 0.5,
                                                stroke_width=size,
                                                stroke=color,
                                                stroke_linecap="round",
                                            ),
                                        )

                            else:
                                if x == 0 or (
                                    x > 0
                                    and (not matrix[x - 1][y] or not ava2[x - 1][y])
                                ):
                                    start = 0
                                    end = 0
                                    ctn = True
                                    while ctn and x + end < n_count:
                                        if (
                                            matrix[x + end][y]
                                            and ava2[x + end][y]
                                            and end - start <= 3
                                        ):
                                            end += 1
                                        else:
                                            ctn = False

                                    if end - start > 1:
                                        for i in range(start, end):
                                            ava2[x + i][y] = False
                                            available[x + i][y] = False

                                        points.append(
                                            line(
                                                opacity=opacity,
                                                x1=x + 0.5,
                                                y1=y + 0.5,
                                                x2=x + end - start - 0.5,
                                                y2=y + 0.5,
                                                stroke_width=size,
                                                stroke=color,
                                                stroke_linecap="round",
                                            ),
                                        )

                            if available[x][y]:
                                points.append(
                                    circle(
                                        opacity=opacity,
                                        r=size / 2.0,
                                        fill=color,
                                        cx=x + 0.5,
                                        cy=y + 0.5,
                                    ),
                                )

                        if self.content_line_type == ContentLineType.tl_br:
                            if (
                                y == 0
                                or x == 0
                                or (
                                    y > 0
                                    and x > 0
                                    and (
                                        not matrix[x - 1][y - 1]
                                        or not ava2[x - 1][y - 1]
                                    )
                                )
                            ):
                                start = 0
                                end = 0
                                ctn = True
                                while ctn and y + end < n_count and x + end < n_count:
                                    if (
                                        matrix[x + end][y + end]
                                        and ava2[x + end][y + end]
                                    ):
                                        end += 1
                                    else:
                                        ctn = False

                                if end - start > 1:
                                    for i in range(start, end):
                                        ava2[x + i][y + i] = False
                                        available[x + i][y + i] = False

                                    points.append(
                                        line(
                                            opacity=opacity,
                                            x1=x + 0.5,
                                            y1=y + 0.5,
                                            x2=x + end - start - 1 + 0.5,
                                            y2=y + end - start - 1 + 0.5,
                                            stroke_width=size,
                                            stroke=color,
                                            stroke_linecap="round",
                                        ),
                                    )

                            if available[x][y]:
                                points.append(
                                    circle(
                                        opacity=opacity,
                                        r=size / 2.0,
                                        fill=color,
                                        cx=x + 0.5,
                                        cy=y + 0.5,
                                    ),
                                )

                        if self.content_line_type == ContentLineType.tr_bl:
                            if (
                                x == 0
                                or y == n_count - 1
                                or (
                                    x > 0
                                    and y < n_count - 1
                                    and (
                                        not matrix[x - 1][y + 1]
                                        or not ava2[x - 1][y + 1]
                                    )
                                )
                            ):
                                start = 0
                                end = 0
                                ctn = True
                                while ctn and x + end < n_count and y - end >= 0:
                                    if (
                                        matrix[x + end][y - end]
                                        and available[x + end][y - end]
                                    ):
                                        end += 1
                                    else:
                                        ctn = False

                                if end - start > 1:
                                    for i in range(start, end):
                                        ava2[x + i][y - i] = False
                                        available[x + i][y - i] = False

                                    points.append(
                                        line(
                                            opacity=opacity,
                                            x1=x + 0.5,
                                            y1=y + 0.5,
                                            x2=x + (end - start - 1) + 0.5,
                                            y2=y - (end - start - 1) + 0.5,
                                            stroke_width=size,
                                            stroke=color,
                                            stroke_linecap="round",
                                        ),
                                    )

                            if available[x][y]:
                                points.append(
                                    circle(
                                        opacity=opacity,
                                        r=size / 2.0,
                                        fill=color,
                                        cx=x + 0.5,
                                        cy=y + 0.5,
                                    ),
                                )

                        if self.content_line_type == ContentLineType.cross:
                            if (
                                x == 0
                                or y == n_count - 1
                                or (
                                    x > 0
                                    and y < n_count - 1
                                    and (
                                        not matrix[x - 1][y + 1]
                                        or not ava2[x - 1][y + 1]
                                    )
                                )
                            ):
                                start = 0
                                end = 0
                                ctn = True
                                while ctn and x + end < n_count and y - end >= 0:
                                    if (
                                        matrix[x + end][y - end]
                                        and ava2[x + end][y - end]
                                    ):
                                        end += 1
                                    else:
                                        ctn = False

                                if end - start > 1:
                                    for i in range(start, end):
                                        ava2[x + i][y - i] = False

                                    points.append(
                                        line(
                                            opacity=opacity,
                                            x1=x + 0.5,
                                            y1=y + 0.5,
                                            x2=x + (end - start - 1) + 0.5,
                                            y2=y - (end - start - 1) + 0.5,
                                            stroke_width=(size / 2.0) * random(0.3, 1),
                                            stroke=color,
                                            stroke_linecap="round",
                                        ),
                                    )

                            if (
                                y == 0
                                or x == 0
                                or (
                                    y > 0
                                    and x > 0
                                    and (
                                        not matrix[x - 1][y - 1]
                                        or not available[x - 1][y - 1]
                                    )
                                )
                            ):
                                start = 0
                                end = 0
                                ctn = True
                                while ctn and y + end < n_count and x + end < n_count:
                                    if (
                                        matrix[x + end][y + end]
                                        and available[x + end][y + end]
                                    ):
                                        end += 1
                                    else:
                                        ctn = False

                                if end - start > 1:
                                    for i in range(start, end):
                                        available[x + i][y + i] = False

                                    points.append(
                                        line(
                                            opacity=opacity,
                                            x1=x + 0.5,
                                            y1=y + 0.5,
                                            x2=x + end - start - 1 + 0.5,
                                            y2=y + end - start - 1 + 0.5,
                                            stroke_width=(size / 2.0) * random(0.3, 1),
                                            stroke=color,
                                            stroke_linecap="round",
                                        ),
                                    )

                            points.append(
                                circle(
                                    opacity=opacity,
                                    r=0.5 * random(0.33, 0.9),
                                    fill=color,
                                    cx=x + 0.5,
                                    cy=y + 0.5,
                                ),
                            )

        return points



__version__ = "0.0.1"

from .circles import CirclesDrawer
from .dots import DotsDrawer
from .lines import LinesDrawer
from .sp1 import SP1Drawer
from .utils.center_module import PositioningPointType
from .utils.type_table import QRPointType



from enum import Enum

from .base_drawer import BaseDrawer
from .const import sq25
from .type_table import QRPointType


class PositioningPointType(str, Enum):
    square = "square"
    circle = "circle"
    planet = "planet"
    rounded = "rounded"
    dsj = "dsj"


class CenterModule(BaseDrawer):
    def __init__(
        self,
        positioning_point_type: PositioningPointType,
        positioning_stroke_width=0.8,
        **kwargs,
    ):
        self.positioning_stroke_width = positioning_stroke_width
        self.positioning_point_type = positioning_point_type
        super().__init__(**kwargs)

    def draw_center(self, points, colors, x, y):
        color_center = colors.get(
            QRPointType.POS_CENTER, colors.get(QRPointType.DATA, "#000")
        )
        color_other = colors.get(
            QRPointType.POS_OTHER,
            colors.get(QRPointType.POS_CENTER, colors.get(QRPointType.DATA, "#000")),
        )

        vw = [3, -3]
        vh = [3, -3]

        if self.positioning_point_type == PositioningPointType.square:
            points.append(
                self.rect(
                    fill=color_center,
                    x=x + 0.5 - 1.5,
                    y=y + 0.5 - 1.5,
                    width=3,
                    height=3,
                ),
            )
            points.append(
                self.rect(
                    fill="none",
                    stroke_width=self.positioning_stroke_width,
                    stroke=color_other,
                    x=x + 0.5 - 3,
                    y=y + 0.5 - 3,
                    width=6,
                    height=6,
                ),
            )
        elif self.positioning_point_type == PositioningPointType.circle:
            points.append(
                self.circle(
                    fill=color_center,
                    cx=x + 0.5,
                    cy=y + 0.5,
                    r=1.5,
                ),
            )
            points.append(
                self.circle(
                    fill="none",
                    stroke_width=self.positioning_stroke_width,
                    stroke=color_other,
                    cx=x + 0.5,
                    cy=y + 0.5,
                    r=3,
                ),
            )
        elif self.positioning_point_type == PositioningPointType.planet:
            points.append(
                self.circle(
                    fill=color_center,
                    cx=x + 0.5,
                    cy=y + 0.5,
                    r=1.5,
                ),
            )
            points.append(
                self.circle(
                    fill="none",
                    stroke_width="0.15",
                    stroke_dasharray="0.5,0.5",
                    stroke=color_other,
                    cx=x + 0.5,
                    cy=y + 0.5,
                    r=3,
                ),
            )
            for w in range(len(vw)):
                points.append(
                    self.circle(
                        fill=color_other,
                        cx=x + vw[w] + 0.5,
                        cy=y + 0.5,
                        r=self.positioning_stroke_width,
                    ),
                )
            for h in range(len(vh)):
                points.append(
                    self.circle(
                        fill=color_other,
                        cx=x + 0.5,
                        cy=y + vh[h] + 0.5,
                        r=self.positioning_stroke_width,
                    ),
                )
        elif self.positioning_point_type == PositioningPointType.rounded:
            points.append(
                self.circle(
                    fill=color_center,
                    cx=x + 0.5,
                    cy=y + 0.5,
                    r=1.5,
                ),
            )
            points.append(
                self.path(
                    d=sq25,
                    stroke=color_other,
                    stroke_width=(100 / 6.0)
                    * (1 - (1 - self.positioning_stroke_width) * 0.75),
                    fill="none",
                    transform=f"translate({x - 2.5} ,{y - 2.5}) scale({6 / 100.} ,{6 / 100.})",
                ),
            )

        elif self.positioning_point_type == PositioningPointType.dsj:
            points.append(
                self.rect(
                    width=3 - (1 - self.positioning_stroke_width),
                    height=3 - (1 - self.positioning_stroke_width),
                    fill=color_center,
                    x=x - 1 + (1 - self.positioning_stroke_width) / 2.0,
                    y=y - 1 + (1 - self.positioning_stroke_width) / 2.0,
                ),
            )
            points.append(
                self.rect(
                    width=self.positioning_stroke_width,
                    height=3 - (1 - self.positioning_stroke_width),
                    fill=color_other,
                    x=x - 3 + (1 - self.positioning_stroke_width) / 2.0,
                    y=y - 1 + (1 - self.positioning_stroke_width) / 2.0,
                ),
            )
            points.append(
                self.rect(
                    width=self.positioning_stroke_width,
                    height=3 - (1 - self.positioning_stroke_width),
                    fill=color_other,
                    x=x + 3 + (1 - self.positioning_stroke_width) / 2.0,
                    y=y - 1 + (1 - self.positioning_stroke_width) / 2.0,
                ),
            )
            points.append(
                self.rect(
                    width=3 - (1 - self.positioning_stroke_width),
                    height=self.positioning_stroke_width,
                    fill=color_other,
                    x=x - 1 + (1 - self.positioning_stroke_width) / 2.0,
                    y=y - 3 + (1 - self.positioning_stroke_width) / 2.0,
                ),
            )
            points.append(
                self.rect(
                    width=3 - (1 - self.positioning_stroke_width),
                    height=self.positioning_stroke_width,
                    fill=color_other,
                    x=x - 1 + (1 - self.positioning_stroke_width) / 2.0,
                    y=y + 3 + (1 - self.positioning_stroke_width) / 2.0,
                ),
            )



from pyqrbtf.utils.center_module import CenterModule
from .utils.type_table import get_type_table, QRPointType


class SP1Drawer(CenterModule):
    def __init__(
        self,
        content_stroke_width=0.7,
        content_x_stroke_width=0.7,
        positioning_stroke_width=0.9,
        **kwargs,
    ):
        self.content_stroke_width = content_stroke_width
        self.content_x_stroke_width = content_x_stroke_width
        self.positioning_stroke_width = positioning_stroke_width
        super().__init__(**kwargs)

    def draw(self, matrix, colors):
        table = matrix
        type_table = get_type_table(table)

        points = []

        n_count = len(table)

        xmark_color = colors.get(QRPointType.TIMING, "#0B2D97")
        xmark_w = self.content_x_stroke_width

        content_color = colors.get(QRPointType.FORMAT, "#E02020")
        content_w = self.content_stroke_width

        important_color = colors.get(QRPointType.DATA, "#F6B506")

        line = self.line
        rect = self.rect

        available = [[True] * n_count for _ in range(n_count)]
        ava2 = [[True] * n_count for _ in range(n_count)]

        g1 = []
        g2 = []

        for y in range(n_count):
            for x in range(n_count):
                if not table[x][y]:
                    continue
                match type_table[x][y]:
                    case QRPointType.POS_OTHER:
                        continue
                    case QRPointType.POS_CENTER:
                        self.draw_center(points, colors, x, y)
                    case _:
                        if (
                            available[x][y]
                            and ava2[x][y]
                            and x < n_count - 2
                            and y < n_count - 2
                        ):
                            ctn = True
                            for i in range(3):
                                for j in range(3):
                                    try:
                                        if ava2[x + i][y + j] == False:
                                            ctn = False
                                    except IndexError:
                                        pass
                            if (
                                ctn
                                and table[x + 2][y]
                                and table[x + 1][y + 1]
                                and table[x][y + 2]
                                and table[x + 2][y + 2]
                            ):
                                g1.append(
                                    line(
                                        x1=x + xmark_w / (8**0.5),
                                        y1=y + xmark_w / (8**0.5),
                                        x2=x + 3 - xmark_w / (8**0.5),
                                        y2=y + 3 - xmark_w / (8**0.5),
                                        fill="none",
                                        stroke=xmark_color,
                                        stroke_width=xmark_w,
                                    ),
                                )
                                g1.append(
                                    line(
                                        x1=x + 3 - xmark_w / (8**0.5),
                                        y1=y + xmark_w / (8**0.5),
                                        x2=x + xmark_w / (8**0.5),
                                        y2=y + 3 - xmark_w / (8**0.5),
                                        fill="none",
                                        stroke=xmark_color,
                                        stroke_width=xmark_w,
                                    ),
                                )
                                available[x][y] = False
                                available[x + 2][y] = False
                                available[x][y + 2] = False
                                available[x + 2][y + 2] = False
                                available[x + 1][y + 1] = False
                                for i in range(3):
                                    for j in range(3):
                                        try:
                                            ava2[x + i][y + j] = False
                                        except IndexError:
                                            pass
                        if (
                            available[x][y]
                            and ava2[x][y]
                            and x < n_count - 1
                            and y < n_count - 1
                        ):
                            ctn = True
                            for i in range(2):
                                for j in range(2):
                                    try:
                                        if ava2[x + i][y + j] == False:
                                            ctn = False
                                    except IndexError:
                                        pass
                            if (
                                ctn
                                and table[x + 1][y]
                                and table[x][y + 1]
                                and table[x + 1][y + 1]
                            ):
                                g1.append(
                                    line(
                                        x1=x + xmark_w / (8**0.5),
                                        y1=y + xmark_w / (8**0.5),
                                        x2=x + 2 - xmark_w / (8**0.5),
                                        y2=y + 2 - xmark_w / (8**0.5),
                                        fill="none",
                                        stroke=xmark_color,
                                        stroke_width=xmark_w,
                                    ),
                                )
                                g1.append(
                                    line(
                                        x1=x + 2 - xmark_w / (8**0.5),
                                        y1=y + xmark_w / (8**0.5),
                                        x2=x + xmark_w / (8**0.5),
                                        y2=y + 2 - xmark_w / (8**0.5),
                                        fill="none",
                                        stroke=xmark_color,
                                        stroke_width=xmark_w,
                                    ),
                                )
                                for i in range(2):
                                    for j in range(2):
                                        try:
                                            available[x + i][y + j] = False
                                            ava2[x + i][y + j] = False
                                        except IndexError:
                                            pass
                        if available[x][y] and ava2[x][y]:
                            if y == 0 or (
                                y > 0 and (not table[x][y - 1] or not ava2[x][y - 1])
                            ):
                                start = y
                                end = y
                                ctn = True
                                while ctn and end < n_count:
                                    if table[x][end] and ava2[x][end]:
                                        end += 1
                                    else:
                                        ctn = False
                                if end - start > 2:
                                    for i in range(start, end):
                                        ava2[x][i] = False
                                        available[x][i] = False
                                    g2.append(
                                        rect(
                                            width=content_w,
                                            height=end - start - 1 - (1 - content_w),
                                            fill=content_color,
                                            x=x + (1 - content_w) / 2.0,
                                            y=y + (1 - content_w) / 2.0,
                                        ),
                                    )
                                    g2.append(
                                        rect(
                                            width=content_w,
                                            height=content_w,
                                            fill=content_color,
                                            x=x + (1 - content_w) / 2.0,
                                            y=end - 1 + (1 - content_w) / 2.0,
                                        ),
                                    )
                        if available[x][y] and ava2[x][y]:
                            if x == 0 or (
                                x > 0 and (not table[x - 1][y] or not ava2[x - 1][y])
                            ):
                                start = x
                                end = x
                                ctn = True
                                while ctn and end < n_count:
                                    if table[end][y] and ava2[end][y]:
                                        end += 1
                                    else:
                                        ctn = False
                                if end - start > 1:
                                    for i in range(start, end):
                                        ava2[i][y] = False
                                        available[i][y] = False
                                    g2.append(
                                        rect(
                                            width=end - start - (1 - content_w),
                                            height=content_w,
                                            fill=important_color,
                                            x=x + (1 - content_w) / 2.0,
                                            y=y + (1 - content_w) / 2.0,
                                        ),
                                    )
                        if available[x][y]:
                            points.append(
                                rect(
                                    width=content_w,
                                    height=content_w,
                                    fill=important_color,
                                    x=x + (1 - content_w) / 2.0,
                                    y=y + (1 - content_w) / 2.0,
                                ),
                            )
        return [*points, *g1, *g2]



import os

from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

_commitTag = os.environ.get("CI_COMMIT_TAG")

setup(
    name="pyqrbtf",
    version=_commitTag or "0.0.0",
    author="Yorsh Siarhei",
    author_email="myrik260138@gmail.com",
    description="A project for generating a QR codes in SVG format",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/Myrik/pyqrbtf",
    # packages=find_packages("src"),
    package_dir={"pyqrbtf": "pyqrbtf"},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    python_requires=">=3.8",
    install_requires=[
        "qrcode",
    ],
)
